@@ -22,6 +22,8 @@
 #ifndef __NEAR_NDEF_H
 #define __NEAR_NDEF_H
 
+#include <stdbool.h>
+
 #include <near/tag.h>
 
 struct near_ndef_record;
@@ -59,9 +61,9 @@ near_ndef_prepare_smartposter_record(uint8_t uri_identifier,
 					uint32_t uri_field_length,
 					uint8_t *uri_field);
 
-near_bool_t near_ndef_record_cmp_id(struct near_ndef_record *rec1,
+bool near_ndef_record_cmp_id(struct near_ndef_record *rec1,
 						struct near_ndef_record *rec2);
-near_bool_t near_ndef_record_cmp_mime(struct near_ndef_record *rec1,
+bool near_ndef_record_cmp_mime(struct near_ndef_record *rec1,
 						struct near_ndef_record *rec2);
 
 
