@@ -30,6 +30,8 @@
 #define NFC_HEADER_SIZE 1
 
 #define NFC_MAX_NFCID1_LEN 10
+#define NFC_MAX_ISO15693_DSFID_LEN 1
+#define NFC_MAX_ISO15693_UID_LEN 8
 
 enum near_tag_sub_type {
 	NEAR_TAG_NFC_T2_MIFARE_ULTRALIGHT = 0,	// SAK 0x00
@@ -91,6 +93,8 @@ uint8_t *near_tag_get_nfcid(uint32_t adapter_idx, uint32_t target_idx,
 					uint8_t *nfcid_len);
 int near_tag_set_nfcid(uint32_t adapter_idx, uint32_t target_idx,
 					uint8_t *nfcid, size_t nfcid_len);
+uint8_t *near_tag_get_iso15693_dsfid(uint32_t adapter_idx, uint32_t target_idx);
+uint8_t *near_tag_get_iso15693_uid(uint32_t adapter_idx, uint32_t target_idx);
 uint8_t *near_tag_get_data(struct near_tag *tag, size_t *data_length);
 size_t near_tag_get_data_length(struct near_tag *tag);
 uint32_t near_tag_get_adapter_idx(struct near_tag *tag);
@@ -104,11 +108,19 @@ void near_tag_set_max_ndef_size(struct near_tag *tag, uint16_t size);
 uint16_t near_tag_get_max_ndef_size(struct near_tag *tag);
 void near_tag_set_c_apdu_max_size(struct near_tag *tag, uint16_t size);
 uint16_t near_tag_get_c_apdu_max_size(struct near_tag *tag);
+void near_tag_set_r_apdu_max_size(struct near_tag *tag, uint16_t size);
+uint16_t near_tag_get_r_apdu_max_size(struct near_tag *tag);
+void near_tag_set_file_id(struct near_tag *tag, uint16_t file_id);
+uint16_t near_tag_get_file_id(struct near_tag *tag);
 void near_tag_set_idm(struct near_tag *tag, uint8_t *idm, uint8_t len);
 uint8_t *near_tag_get_idm(struct near_tag *tag, uint8_t *len);
 void near_tag_set_attr_block(struct near_tag *tag, uint8_t *attr, uint8_t len);
 uint8_t *near_tag_get_attr_block(struct near_tag *tag, uint8_t *len);
 void near_tag_set_ic_type(struct near_tag *tag, uint8_t ic_type);
 uint8_t near_tag_get_ic_type(struct near_tag *tag);
+uint8_t near_tag_get_blk_size(struct near_tag *tag);
+void near_tag_set_blk_size(struct near_tag *tag, uint8_t blk_size);
+uint8_t near_tag_get_num_blks(struct near_tag *tag);
+void near_tag_set_num_blks(struct near_tag *tag, uint8_t num_blks);
 
 #endif
